from wkregister.producer import KafkaProducerWrapper, KafkaMessageSender
from wkregister.models import Records
from wkregister.util import kafkaParams
from typing import Callable, Any, Dict
from functools import wraps
import inspect
import asyncio
import json
import time


async def record2Kafka(org: str, key: str, value: Dict) -> None:
    producer_wrapper = KafkaProducerWrapper()
    # Crear el enviador de mensajes
    message_sender = KafkaMessageSender(producer_wrapper)
    # Enviar un mensaje
    topic = f"{org}_logs"
    key = f"{key}"
    value = json.dumps(value)
    await message_sender.send(topic, key, value)

    return None


# Decorator for logging
def record():
    def decorator_log(func: Callable):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            input = func(*args, **kwargs)

            if inspect.iscoroutine(input):
                result = await input
            else:
                result = input

            org, key, log = kafkaParams(result)
            print(f"log={log.dict()}")

            start_time = time.time()  # Capture the start time

            # Log after function execution, you can adjust what you log as needed
            asyncio.create_task(record2Kafka(org, key, log.dict()))

            end_time = time.time()  # Capture the end time

            elapsed_time = end_time - start_time  # Calculate elapsed time
            # print(f"The function took {elapsed_time} seconds to complete.")

            output = {}
            for key, value in result.items():
                if not key == "record" or not key == "log":
                    output[key] = value
                if key == "record" or key == "log":
                    output[key] = log.dict()

            print(output)

            return output

        return wrapper

    return decorator_log



from wkregister.models import Records
from typing import Callable, Any
import json


def kafkaParams(result: Any):

    # Retrieve 'record' or 'log', defaulting to None if neither is found
    log: Records = (result).get("record") or (result).get("log")
    # Check if 'log' was not retrieved and inform the user
    if not log:
        print("No record/log was set")

    # Extract 'org' and 'key' from the log, defaulting to None if not found
    org = log.org
    key = log.key

    # Check if 'org' or 'key' was not set and inform the user
    if not org:
        print("Org was not set")
    if not key:
        print("Key was not set")

    # Return the extracted values
    return org, key, log



import os
from confluent_kafka import Producer, KafkaException
from dotenv import load_dotenv

load_dotenv()


producerconf = {
    "bootstrap.servers": os.getenv("BOOTSTRAP_SERVER"),
    "security.protocol": os.getenv("SECURITY_PROTOCOL"),
    "sasl.mechanisms": os.getenv("SASL_MECHANISM"),
    "sasl.username": os.getenv("SASL_USERNAME"),
    "sasl.password": os.getenv("SAS_PASSWORD"),
}


class KafkaProducerWrapper:
    def __init__(self):
        self.producer = Producer(producerconf)

    def get_producer(self):
        return self.producer


class KafkaMessageSender:
    def __init__(self, producer_wrapper):
        self.producer = producer_wrapper.get_producer()

    async def send(self, topic, key, value):
        def delivery_report(err, msg):
            if err is not None:
                print(f"Message not sent: {err}")
            else:
                # print(f'Message sent: {msg.value()}')
                print(f"Message sent")

        try:
            # Intenta enviar el mensaje
            self.producer.produce(topic, key=key, value=value, callback=delivery_report)
            self.producer.flush()  # Espera a que el mensaje se entregue
            return True  # Retorna True si se entregó exitosamente
        except KafkaException as e:
            print(f"Error sending to Kafka: {e}")
            return False  # Retorna False si ocurrió algún error



from dataclasses import dataclass, field, asdict
from typing import Dict
import uuid
from datetime import datetime


@dataclass
class Records:
    org: str = ""
    key: str = ""
    userId: str = ""
    actionType: str = ""
    status: str = ""
    errorMessage: str = ""
    service: str = ""
    id: str = field(default_factory=lambda: str(uuid.uuid4()))
    timestamp: str = field(default_factory=lambda: str(datetime.today()))
    payload: Dict = field(default_factory=dict)

    def dict(self):
        return {k: str(v) for k, v in asdict(self).items()}
